@@ -24,6 +24,7 @@
 #include <sys/types.h>
 #include "ruby.h"
 #include "ruby/io.h"
+#include "rmagick_gvl.h"
 
 #if defined(__MINGW32__)
     // Ruby defines wrong format specifiers for MinGW. So this defines original macro in here.
@@ -61,6 +62,11 @@
 #undef PACKAGE_TARNAME
 #undef WORDS_BIGENDIAN
 
+#ifndef HAVE_RB_EXT_RACTOR_SAFE
+#undef RUBY_TYPED_FROZEN_SHAREABLE
+#define RUBY_TYPED_FROZEN_SHAREABLE 0
+#endif
+
 #include "extconf.h"
 
 #if defined(IMAGEMAGICK_7)
@@ -90,8 +96,8 @@
 
 //! Trace new image creation in bang methods
 #define UPDATE_DATA_PTR(_obj_, _new_) \
-    do { (void) rm_trace_creation(_new_);\
-    DATA_PTR(_obj_) = (void *)(_new_);\
+    do { \
+        DATA_PTR(_obj_) = (void *)(_new_); \
     } while(0)
 
 
@@ -396,7 +402,6 @@ EXTERN VALUE Class_KernelInfoType;
 /**
 *   Commonly-used IDs
 */
-EXTERN ID rm_ID_trace_proc;        /**< "@trace_proc" */
 EXTERN ID rm_ID_call;              /**< "call" */
 EXTERN ID rm_ID_changed;           /**< "changed" */
 EXTERN ID rm_ID_cur_image;         /**< "cur_image" */
@@ -411,7 +416,13 @@ EXTERN ID rm_ID_push;              /**< "push" */
 EXTERN ID rm_ID_values;            /**< "values" */
 EXTERN ID rm_ID_width;             /**< "width" */
 
-EXTERN unsigned long long rm_main_thread_id;
+extern const rb_data_type_t rm_enum_data_type;
+extern const rb_data_type_t rm_info_data_type;
+extern const rb_data_type_t rm_image_data_type;
+extern const rb_data_type_t rm_draw_data_type;
+extern const rb_data_type_t rm_pixel_data_type;
+extern const rb_data_type_t rm_montage_data_type;
+extern const rb_data_type_t rm_kernel_info_data_type;
 
 #if !defined(min)
 #define min(a, b) ((a)<(b)?(a):(b)) /**< min of two values */
@@ -434,56 +445,56 @@ EXTERN unsigned long long rm_main_thread_id;
 #define R_boolean_to_C_boolean(attr) RTEST(attr) /**<  C boolean <- Ruby boolean */
 #define C_int_to_R_int(attr) INT2FIX(attr) /**< C int -> Ruby int */
 #define R_int_to_C_int(attr) NUM2INT(attr) /**< C int <- Ruby int */
-#define C_long_to_R_long(attr) INT2NUM(attr) /**< C long -> Ruby long */
+#define C_long_to_R_long(attr) LONG2NUM(attr) /**< C long -> Ruby long */
 #define R_long_to_C_long(attr) NUM2LONG(attr) /**< C long <- Ruby long */
-#define C_ulong_to_R_ulong(attr) UINT2NUM(attr) /**< C unsigned long -> Ruby unsigned long */
+#define C_ulong_to_R_ulong(attr) ULONG2NUM(attr) /**< C unsigned long -> Ruby unsigned long */
 #define R_ulong_to_C_ulong(attr) NUM2ULONG(attr) /**< C unsigned long <- Ruby unsigned long */
 #define C_str_to_R_str(attr) attr ? rb_str_new2(attr) : Qnil /**< C string -> Ruby string */
 #define C_dbl_to_R_dbl(attr) rb_float_new(attr) /**< C double -> Ruby double */
 #define R_dbl_to_C_dbl(attr) NUM2DBL(attr) /**< C double <- Ruby double */
 
 //! define attribute reader
-#define IMPLEMENT_ATTR_READER(class, attr, type) \
+#define IMPLEMENT_TYPED_ATTR_READER(class, attr, type, data_type) \
     {\
         class *ptr;\
         if (rb_obj_is_kind_of(self, Class_Image) == Qtrue) {\
             rm_check_destroyed(self); \
         }\
-        Data_Get_Struct(self, class, ptr);\
+        TypedData_Get_Struct(self, class, data_type, ptr);\
         return C_##type##_to_R_##type(ptr->attr);\
     }
 
 //! define attribute reader when attribute name is different from the field name
-#define IMPLEMENT_ATTR_READERF(class, attr, field, type) \
+#define IMPLEMENT_TYPED_ATTR_READERF(class, attr, field, type, data_type) \
     {\
         class *ptr;\
         rm_check_destroyed(self); \
-        Data_Get_Struct(self, class, ptr);\
+        TypedData_Get_Struct(self, class, data_type, ptr);\
         return C_##type##_to_R_##type(ptr->field);\
     }
 
 //! define attribute writer
-#define IMPLEMENT_ATTR_WRITER(class, attr, type) \
+#define IMPLEMENT_TYPED_ATTR_WRITER(class, attr, type, data_type) \
     {\
         class *ptr;\
         if (rb_obj_is_kind_of(self, Class_Image) == Qtrue) {\
             rm_check_destroyed(self); \
         }\
         rb_check_frozen(self);\
-        Data_Get_Struct(self, class, ptr);\
+        TypedData_Get_Struct(self, class, data_type, ptr);\
         ptr->attr = R_##type##_to_C_##type(val);\
         return val;\
     }
 
 //! define attribute writer when attribute name is different from the field name
-#define IMPLEMENT_ATTR_WRITERF(class, attr, field, type) \
+#define IMPLEMENT_TYPED_ATTR_WRITERF(class, attr, field, type, data_type) \
     {\
         class *ptr;\
         if (rb_obj_is_kind_of(self, Class_Image) == Qtrue) {\
             rm_check_destroyed(self); \
         }\
         rb_check_frozen(self);\
-        Data_Get_Struct(self, class, ptr);\
+        TypedData_Get_Struct(self, class, data_type, ptr);\
         ptr->field = R_##type##_to_C_##type(val);\
         return self;\
     }
@@ -513,7 +524,6 @@ EXTERN unsigned long long rm_main_thread_id;
 #define DEF_CONSTV(constant, val) rb_define_const(Module_Magick, #constant, UINT2NUM(val))
 #endif
 
-
 //! Convert a Ruby enum constant back to a C enum member.
 #define VALUE_TO_ENUM(value, e, type) \
    do {\
@@ -521,7 +531,7 @@ EXTERN unsigned long long rm_main_thread_id;
    if (CLASS_OF(value) != Class_##type)\
        rb_raise(rb_eTypeError, "wrong enumeration type - expected %s, got %s", \
                 rb_class2name(Class_##type), rb_class2name(CLASS_OF(value)));\
-   Data_Get_Struct(value, MagickEnum, magick_enum);\
+   TypedData_Get_Struct(value, MagickEnum, &rm_enum_data_type, magick_enum);\
    e = (type)(magick_enum->val);\
    } while(0)
 
@@ -1019,7 +1029,6 @@ extern VALUE Image_write(VALUE, VALUE);
 
 extern VALUE rm_image_new(Image *);
 extern void  rm_image_destroy(void *);
-extern void  rm_trace_creation(Image *);
 
 
 // rmfill.c
@@ -1043,7 +1052,6 @@ ATTR_ACCESSOR(Pixel, cyan)
 ATTR_ACCESSOR(Pixel, magenta)
 ATTR_ACCESSOR(Pixel, yellow)
 ATTR_ACCESSOR(Pixel, black)
-extern void   destroy_Pixel(Pixel *);
 extern VALUE  Pixel_alloc(VALUE);
 extern VALUE  Pixel_case_eq(VALUE, VALUE);
 extern VALUE  Pixel_clone(VALUE);
@@ -1193,7 +1201,6 @@ typedef enum
 extern void   rm_check_exception(ExceptionInfo *, Image *, ErrorRetention);
 extern void   rm_ensure_result(Image *);
 extern Image *rm_clone_image(Image *);
-extern MagickBooleanType rm_progress_monitor(const char *, const MagickOffsetType, const MagickSizeType, void *);
 extern VALUE  rm_exif_by_entry(Image *);
 extern VALUE  rm_exif_by_number(Image *);
 extern void   rm_get_optional_arguments(VALUE);
@@ -1202,7 +1209,6 @@ extern void   rm_error_handler(const ExceptionType, const char *, const char *);
 extern void   rm_warning_handler(const ExceptionType, const char *, const char *);
 extern MagickBooleanType rm_should_raise_exception(ExceptionInfo *, const ExceptionRetention);
 extern void   rm_raise_exception(ExceptionInfo *);
-extern unsigned long long rm_current_thread_id();
 #if defined(IMAGEMAGICK_6)
 extern void   rm_check_image_exception(Image *, ErrorRetention);
 #endif
