@@ -13,17 +13,43 @@
 #include "rmagick.h"
 #include "float.h"
 
-static void mark_Draw(void *);
-static void destroy_Draw(void *);
+#ifdef HAVE_RB_GC_MARK_MOVABLE
+static void Draw_compact(void *drawptr);
+#endif
+static void Draw_mark(void *);
+static void Draw_destroy(void *);
+static size_t Draw_memsize(const void *);
 static VALUE new_DrawOptions(void);
+static VALUE get_type_metrics(int, VALUE *, VALUE, gvl_function_t);
 
-/** Method that gets type metrics */
+const rb_data_type_t rm_draw_data_type = {
+    "Magick::Draw",
+    {
+        Draw_mark,
+        Draw_destroy,
+        Draw_memsize,
+#ifdef HAVE_RB_GC_MARK_MOVABLE
+        Draw_compact,
+#endif
+    },
+    0, 0,
+    RUBY_TYPED_FROZEN_SHAREABLE,
+};
+
+
+DEFINE_GVL_STUB4(BlobToImage, const ImageInfo *, const void *, const size_t, ExceptionInfo *);
+DEFINE_GVL_STUB4(ImageToBlob, const ImageInfo *, Image *, size_t *, ExceptionInfo *);
 #if defined(IMAGEMAGICK_7)
-    typedef MagickBooleanType (get_type_metrics_func_t)(Image *, const DrawInfo *, TypeMetric *, ExceptionInfo *);
+DEFINE_GVL_STUB3(AnnotateImage, Image *, const DrawInfo *, ExceptionInfo *);
+DEFINE_GVL_STUB3(DrawImage, Image *, const DrawInfo *, ExceptionInfo *);
+DEFINE_GVL_STUB4(GetMultilineTypeMetrics, Image *, const DrawInfo *, TypeMetric *, ExceptionInfo *);
+DEFINE_GVL_STUB4(GetTypeMetrics, Image *, const DrawInfo *, TypeMetric *, ExceptionInfo *);
 #else
-    typedef MagickBooleanType (get_type_metrics_func_t)(Image *, const DrawInfo *, TypeMetric *);
+DEFINE_GVL_STUB2(AnnotateImage, Image *, const DrawInfo *);
+DEFINE_GVL_STUB2(DrawImage, Image *, const DrawInfo *);
+DEFINE_GVL_STUB3(GetMultilineTypeMetrics, Image *, const DrawInfo *, TypeMetric *);
+DEFINE_GVL_STUB3(GetTypeMetrics, Image *, const DrawInfo *, TypeMetric *);
 #endif
-static VALUE get_type_metrics(int, VALUE *, VALUE, get_type_metrics_func_t);
 
 
 /**
@@ -38,7 +64,7 @@ Draw_affine_eq(VALUE self, VALUE matrix)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     Export_AffineMatrix(&draw->info->affine, matrix);
     return matrix;
 }
@@ -56,7 +82,7 @@ Draw_align_eq(VALUE self, VALUE align)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     VALUE_TO_ENUM(align, draw->info->align, AlignType);
     return align;
 }
@@ -74,7 +100,7 @@ Draw_decorate_eq(VALUE self, VALUE decorate)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     VALUE_TO_ENUM(decorate, draw->info->decorate, DecorationType);
     return decorate;
 }
@@ -92,7 +118,7 @@ Draw_density_eq(VALUE self, VALUE density)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     magick_clone_string(&draw->info->density, StringValueCStr(density));
 
     return density;
@@ -111,7 +137,7 @@ Draw_encoding_eq(VALUE self, VALUE encoding)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     magick_clone_string(&draw->info->encoding, StringValueCStr(encoding));
 
     return encoding;
@@ -130,7 +156,7 @@ Draw_fill_eq(VALUE self, VALUE fill)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     Color_to_PixelColor(&draw->info->fill, fill);
     return fill;
 }
@@ -151,7 +177,7 @@ Draw_fill_pattern_eq(VALUE self, VALUE pattern)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     if (draw->info->fill_pattern != NULL)
     {
@@ -186,7 +212,7 @@ Draw_font_eq(VALUE self, VALUE font)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     magick_clone_string(&draw->info->font, StringValueCStr(font));
 
     return font;
@@ -205,7 +231,7 @@ Draw_font_family_eq(VALUE self, VALUE family)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     magick_clone_string(&draw->info->family, StringValueCStr(family));
 
     return family;
@@ -224,7 +250,7 @@ Draw_font_stretch_eq(VALUE self, VALUE stretch)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     VALUE_TO_ENUM(stretch, draw->info->stretch, StretchType);
     return stretch;
 }
@@ -242,7 +268,7 @@ Draw_font_style_eq(VALUE self, VALUE style)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     VALUE_TO_ENUM(style, draw->info->style, StyleType);
     return style;
 }
@@ -262,7 +288,7 @@ Draw_font_weight_eq(VALUE self, VALUE weight)
     size_t w;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     if (FIXNUM_P(weight))
     {
@@ -328,7 +354,7 @@ Draw_gravity_eq(VALUE self, VALUE grav)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     VALUE_TO_ENUM(grav, draw->info->gravity, GravityType);
 
     return grav;
@@ -347,7 +373,7 @@ Draw_kerning_eq(VALUE self, VALUE kerning)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     draw->info->kerning = NUM2DBL(kerning);
     return kerning;
 }
@@ -365,7 +391,7 @@ Draw_interline_spacing_eq(VALUE self, VALUE spacing)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     draw->info->interline_spacing = NUM2DBL(spacing);
     return spacing;
 }
@@ -383,7 +409,7 @@ Draw_interword_spacing_eq(VALUE self, VALUE spacing)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     draw->info->interword_spacing = NUM2DBL(spacing);
     return spacing;
 }
@@ -415,7 +441,8 @@ image_to_str(Image *image)
 
         info = CloneImageInfo(NULL);
         exception = AcquireExceptionInfo();
-        blob = ImageToBlob(info, image, &length, exception);
+        GVL_STRUCT_TYPE(ImageToBlob) args = { info, image, &length, exception };
+        blob = (unsigned char *)CALL_FUNC_WITHOUT_GVL(GVL_FUNC(ImageToBlob), &args);
         DestroyImageInfo(info);
         CHECK_EXCEPTION();
         DestroyExceptionInfo(exception);
@@ -453,7 +480,8 @@ Image *str_to_image(VALUE str)
 
         info = CloneImageInfo(NULL);
         exception = AcquireExceptionInfo();
-        image = BlobToImage(info, RSTRING_PTR(str), RSTRING_LEN(str), exception);
+        GVL_STRUCT_TYPE(BlobToImage) args = { info, RSTRING_PTR(str), RSTRING_LEN(str), exception };
+        image = (Image *)CALL_FUNC_WITHOUT_GVL(GVL_FUNC(BlobToImage), &args);
         DestroyImageInfo(info);
         CHECK_EXCEPTION();
         DestroyExceptionInfo(exception);
@@ -482,7 +510,7 @@ Draw_marshal_dump(VALUE self)
     Draw *draw;
     VALUE ddraw;
 
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     // Raise an exception if the Draw has a non-NULL gradient or element_reference field
     if (draw->info->element_reference.type != UndefinedReference
@@ -493,31 +521,17 @@ Draw_marshal_dump(VALUE self)
 
     ddraw = rb_hash_new();
 
-    // rb_hash_aset(ddraw, CSTR2SYM("primitive"), MAGICK_STRING_TO_OBJ(draw->info->primitive)); internal
-    // rb_hash_aset(ddraw, CSTR2SYM("geometry"), MAGICK_STRING_TO_OBJ(draw->info->geometry)); set by "text" primitive
-    // rb_hash_aset(ddraw, CSTR2SYM("viewbox"), Import_RectangleInfo(&draw->info->viewbox)); internal
     rb_hash_aset(ddraw, CSTR2SYM("affine"), Import_AffineMatrix(&draw->info->affine));
     rb_hash_aset(ddraw, CSTR2SYM("gravity"), INT2FIX(draw->info->gravity));
     rb_hash_aset(ddraw, CSTR2SYM("fill"), Pixel_from_PixelColor(&draw->info->fill));
     rb_hash_aset(ddraw, CSTR2SYM("stroke"), Pixel_from_PixelColor(&draw->info->stroke));
     rb_hash_aset(ddraw, CSTR2SYM("stroke_width"), rb_float_new(draw->info->stroke_width));
-    // rb_hash_aset(ddraw, CSTR2SYM("gradient"), Qnil);  // not used yet
     rb_hash_aset(ddraw, CSTR2SYM("fill_pattern"), image_to_str(draw->info->fill_pattern));
-    rb_hash_aset(ddraw, CSTR2SYM("tile"), Qnil); // deprecated
     rb_hash_aset(ddraw, CSTR2SYM("stroke_pattern"), image_to_str(draw->info->stroke_pattern));
     rb_hash_aset(ddraw, CSTR2SYM("stroke_antialias"), draw->info->stroke_antialias ? Qtrue : Qfalse);
     rb_hash_aset(ddraw, CSTR2SYM("text_antialias"), draw->info->text_antialias ? Qtrue : Qfalse);
-    // rb_hash_aset(ddraw, CSTR2SYM("fill_rule"), INT2FIX(draw->info->fill_rule)); internal
-    // rb_hash_aset(ddraw, CSTR2SYM("linecap"), INT2FIX(draw->info->linecap));
-    // rb_hash_aset(ddraw, CSTR2SYM("linejoin"), INT2FIX(draw->info->linejoin));
-    // rb_hash_aset(ddraw, CSTR2SYM("miterlimit"), ULONG2NUM(draw->info->miterlimit));
-    // rb_hash_aset(ddraw, CSTR2SYM("dash_offset"), rb_float_new(draw->info->dash_offset));
     rb_hash_aset(ddraw, CSTR2SYM("decorate"), INT2FIX(draw->info->decorate));
-    // rb_hash_aset(ddraw, CSTR2SYM("compose"), INT2FIX(draw->info->compose)); set via "image" primitive
-    // rb_hash_aset(ddraw, CSTR2SYM("text"), MAGICK_STRING_TO_OBJ(draw->info->text)); set via "text" primitive
-    // rb_hash_aset(ddraw, CSTR2SYM("face"), Qnil);  internal
     rb_hash_aset(ddraw, CSTR2SYM("font"), MAGICK_STRING_TO_OBJ(draw->info->font));
-    // rb_hash_aset(ddraw, CSTR2SYM("metrics"), Qnil);   internal
     rb_hash_aset(ddraw, CSTR2SYM("family"), MAGICK_STRING_TO_OBJ(draw->info->family));
     rb_hash_aset(ddraw, CSTR2SYM("style"), INT2FIX(draw->info->style));
     rb_hash_aset(ddraw, CSTR2SYM("stretch"), INT2FIX(draw->info->stretch));
@@ -527,26 +541,17 @@ Draw_marshal_dump(VALUE self)
     rb_hash_aset(ddraw, CSTR2SYM("density"), MAGICK_STRING_TO_OBJ(draw->info->density));
     rb_hash_aset(ddraw, CSTR2SYM("align"), INT2FIX(draw->info->align));
     rb_hash_aset(ddraw, CSTR2SYM("undercolor"), Pixel_from_PixelColor(&draw->info->undercolor));
-    // rb_hash_aset(ddraw, CSTR2SYM("border_color"), Pixel_from_PixelColor(&draw->info->border_color)); Montage and Polaroid
-    // rb_hash_aset(ddraw, CSTR2SYM("server_name"), MAGICK_STRING_TO_OBJ(draw->info->server_name));
-    // rb_hash_aset(ddraw, CSTR2SYM("dash_pattern"), dash_pattern_to_array(draw->info->dash_pattern)); internal
-    // rb_hash_aset(ddraw, CSTR2SYM("clip_mask"), MAGICK_STRING_TO_OBJ(draw->info->clip_mask)); internal
-    // rb_hash_aset(ddraw, CSTR2SYM("bounds"), Import_SegmentInfo(&draw->info->bounds)); internal
     rb_hash_aset(ddraw, CSTR2SYM("clip_units"), INT2FIX(draw->info->clip_units));
 #if defined(IMAGEMAGICK_7)
     rb_hash_aset(ddraw, CSTR2SYM("alpha"), QUANTUM2NUM(draw->info->alpha));
 #else
     rb_hash_aset(ddraw, CSTR2SYM("opacity"), QUANTUM2NUM(draw->info->opacity));
 #endif
-    // rb_hash_aset(ddraw, CSTR2SYM("render"), draw->info->render ? Qtrue : Qfalse); internal
-    // rb_hash_aset(ddraw, CSTR2SYM("element_reference"), Qnil);     // not used yet
-    // rb_hash_aset(ddraw, CSTR2SYM("debug"), draw->info->debug ? Qtrue : Qfalse);
     rb_hash_aset(ddraw, CSTR2SYM("kerning"), rb_float_new(draw->info->kerning));
     rb_hash_aset(ddraw, CSTR2SYM("interword_spacing"), rb_float_new(draw->info->interword_spacing));
 
     // Non-DrawInfo fields
     rb_hash_aset(ddraw, CSTR2SYM("primitives"), draw->primitives);
-    // rb_hash_aset(ddraw, CSTR2SYM("shadow_color"), Pixel_from_PixelColor(&draw->shadow_color)); Polaroid-only
 
     return ddraw;
 }
@@ -564,21 +569,18 @@ Draw_marshal_load(VALUE self, VALUE ddraw)
     Draw *draw;
     VALUE val;
 
-    Data_Get_Struct(self, Draw, draw);
-    
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
+
     if (draw->info == NULL)
     {
         ImageInfo *image_info;
 
         image_info = CloneImageInfo(NULL);
         draw->info = CloneDrawInfo(image_info, (DrawInfo *) NULL);
-        DestroyImageInfo(image_info);        
+        DestroyImageInfo(image_info);
     }
     OBJ_TO_MAGICK_STRING(draw->info->geometry, rb_hash_aref(ddraw, CSTR2SYM("geometry")));
 
-    //val = rb_hash_aref(ddraw, CSTR2SYM("viewbox"));
-    //Export_RectangleInfo(&draw->info->viewbox, val);
-
     val = rb_hash_aref(ddraw, CSTR2SYM("affine"));
     Export_AffineMatrix(&draw->info->affine, val);
 
@@ -639,7 +641,7 @@ Draw_pointsize_eq(VALUE self, VALUE pointsize)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     draw->info->pointsize = NUM2DBL(pointsize);
     return pointsize;
 }
@@ -659,7 +661,7 @@ Draw_rotation_eq(VALUE self, VALUE deg)
     AffineMatrix affine, current;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     degrees = NUM2DBL(deg);
     if (fabs(degrees) > DBL_EPSILON)
@@ -695,7 +697,7 @@ Draw_stroke_eq(VALUE self, VALUE stroke)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     Color_to_PixelColor(&draw->info->stroke, stroke);
     return stroke;
 }
@@ -715,7 +717,7 @@ Draw_stroke_pattern_eq(VALUE self, VALUE pattern)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     if (draw->info->stroke_pattern != NULL)
     {
@@ -751,7 +753,7 @@ Draw_stroke_width_eq(VALUE self, VALUE stroke_width)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     draw->info->stroke_width = NUM2DBL(stroke_width);
     return stroke_width;
 }
@@ -769,7 +771,7 @@ Draw_text_antialias_eq(VALUE self, VALUE text_antialias)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     draw->info->text_antialias = (MagickBooleanType) RTEST(text_antialias);
     return text_antialias;
 }
@@ -800,7 +802,7 @@ Draw_undercolor_eq(VALUE self, VALUE undercolor)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     Color_to_PixelColor(&draw->info->undercolor, undercolor);
     return undercolor;
 }
@@ -843,7 +845,7 @@ VALUE Draw_annotate(
 
     // Save the affine matrix in case it is modified by
     // Draw#rotation=
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     keep = draw->info->affine;
 
     image_arg = rm_cur_image(image_arg);
@@ -853,16 +855,7 @@ VALUE Draw_annotate(
     // allowing the app a chance to modify the object's attributes
     if (rb_block_given_p())
     {
-        if (rb_proc_arity(rb_block_proc()) == 0)
-        {
-            // Run the block in self's context
-            rb_warn("passing a block without an image argument is deprecated");
-            rb_obj_instance_eval(0, NULL, self);
-        }
-        else
-        {
-            rb_yield(self);
-        }
+        rb_yield(self);
     }
 
     // Translate & store in Draw structure
@@ -910,10 +903,11 @@ VALUE Draw_annotate(
     magick_clone_string(&draw->info->geometry, geometry_str);
 
 #if defined(IMAGEMAGICK_7)
-    AnnotateImage(image, draw->info, exception);
+    GVL_STRUCT_TYPE(AnnotateImage) args = { image, draw->info, exception };
 #else
-    AnnotateImage(image, draw->info);
+    GVL_STRUCT_TYPE(AnnotateImage) args = { image, draw->info };
 #endif
+    CALL_FUNC_WITHOUT_GVL(GVL_FUNC(AnnotateImage), &args);
 
     magick_free(draw->info->text);
     draw->info->text = NULL;
@@ -973,7 +967,7 @@ Draw_clone(VALUE self)
  *   @param image [Magick::Image, Magick::ImageList] Either an imagelist or an image. If an
  *     imagelist, uses the current image.
  *   @param operator [Magick::CompositeOperator] the operator
- * 
+ *
  * @return [Magick::Draw] self
  */
 VALUE
@@ -1016,7 +1010,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
         rb_raise(rb_eArgError, "unknown composite operator (%d)", cop);
     }
 
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     // Create a temp copy of the composite image
     rm_write_temp_image(comp_img, name, sizeof(name));
@@ -1061,7 +1055,7 @@ Draw_draw(VALUE self, VALUE image_arg)
     image_arg = rm_cur_image(image_arg);
     image = rm_check_frozen(image_arg);
 
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     if (draw->primitives == 0)
     {
         rb_raise(rb_eArgError, "nothing to draw");
@@ -1072,10 +1066,11 @@ Draw_draw(VALUE self, VALUE image_arg)
 
 #if defined(IMAGEMAGICK_7)
     exception = AcquireExceptionInfo();
-    DrawImage(image, draw->info, exception);
+    GVL_STRUCT_TYPE(DrawImage) args = { image, draw->info, exception };
 #else
-    DrawImage(image, draw->info);
+    GVL_STRUCT_TYPE(DrawImage) args = { image, draw->info };
 #endif
+    CALL_FUNC_WITHOUT_GVL(GVL_FUNC(DrawImage), &args);
 
     magick_free(draw->info->primitive);
     draw->info->primitive = NULL;
@@ -1106,7 +1101,7 @@ Draw_dup(VALUE self)
 
     draw = ALLOC(Draw);
     memset(draw, 0, sizeof(Draw));
-    dup = Data_Wrap_Struct(CLASS_OF(self), mark_Draw, destroy_Draw, draw);
+    dup = TypedData_Wrap_Struct(CLASS_OF(self), &rm_draw_data_type, draw);
     RB_GC_GUARD(dup);
 
     return rb_funcall(dup, rm_ID_initialize_copy, 1, self);
@@ -1136,7 +1131,7 @@ Draw_get_type_metrics(
                      VALUE *argv,
                      VALUE self)
 {
-    return get_type_metrics(argc, argv, self, GetTypeMetrics);
+    return get_type_metrics(argc, argv, self, GVL_FUNC(GetTypeMetrics));
 }
 
 
@@ -1163,7 +1158,7 @@ Draw_get_multiline_type_metrics(
                                VALUE *argv,
                                VALUE self)
 {
-    return get_type_metrics(argc, argv, self, GetMultilineTypeMetrics);
+    return get_type_metrics(argc, argv, self, GVL_FUNC(GetMultilineTypeMetrics));
 }
 
 
@@ -1177,8 +1172,8 @@ VALUE Draw_init_copy(VALUE self, VALUE orig)
 {
     Draw *copy, *original;
 
-    Data_Get_Struct(orig, Draw, original);
-    Data_Get_Struct(self, Draw, copy);
+    TypedData_Get_Struct(orig, Draw, &rm_draw_data_type, original);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, copy);
 
     copy->info = CloneDrawInfo(NULL, original->info);
     if (!copy->info)
@@ -1206,10 +1201,10 @@ Draw_initialize(VALUE self)
     Draw *draw, *draw_options;
     VALUE options;
 
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     options = new_DrawOptions();
-    Data_Get_Struct(options, Draw, draw_options);
+    TypedData_Get_Struct(options, Draw, &rm_draw_data_type, draw_options);
     draw->info = draw_options->info;
     draw_options->info = NULL;
 
@@ -1230,7 +1225,7 @@ Draw_inspect(VALUE self)
 {
     Draw *draw;
 
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     return draw->primitives ? draw->primitives : rb_str_new2("(no primitives defined)");
 }
 
@@ -1247,7 +1242,7 @@ VALUE Draw_alloc(VALUE class)
 
     draw = ALLOC(Draw);
     memset(draw, 0, sizeof(Draw));
-    draw_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw);
+    draw_obj = TypedData_Wrap_Struct(class, &rm_draw_data_type, draw);
 
     RB_GC_GUARD(draw_obj);
 
@@ -1267,7 +1262,7 @@ Draw_primitive(VALUE self, VALUE primitive)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     if (draw->primitives == (VALUE)0)
     {
@@ -1275,13 +1270,32 @@ Draw_primitive(VALUE self, VALUE primitive)
     }
     else
     {
-        draw->primitives = rb_str_concat(draw->primitives, rb_str_new2("\n"));
+        draw->primitives = rb_str_plus(draw->primitives, rb_str_new2("\n"));
         draw->primitives = rb_str_concat(draw->primitives, primitive);
     }
 
     return self;
 }
 
+#ifdef HAVE_RB_GC_MARK_MOVABLE
+/**
+ * Compact the objects.
+ *
+ * No Ruby usage (internal function)
+ *
+ * @param drawptr pointer to a Draw object
+ */
+static void
+Draw_compact(void *drawptr)
+{
+    Draw *draw = (Draw *)drawptr;
+
+    if (draw->primitives != (VALUE)0)
+    {
+        draw->primitives = rb_gc_location(draw->primitives);
+    }
+}
+#endif
 
 /**
  * Mark referenced objects.
@@ -1291,13 +1305,17 @@ Draw_primitive(VALUE self, VALUE primitive)
  * @param drawptr pointer to a Draw object
  */
 static void
-mark_Draw(void *drawptr)
+Draw_mark(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
 
     if (draw->primitives != (VALUE)0)
     {
+#ifdef HAVE_RB_GC_MARK_MOVABLE
+        rb_gc_mark_movable(draw->primitives);
+#else
         rb_gc_mark(draw->primitives);
+#endif
     }
 }
 
@@ -1310,7 +1328,7 @@ mark_Draw(void *drawptr)
  * @param drawptr pointer to a Draw object
  */
 static void
-destroy_Draw(void *drawptr)
+Draw_destroy(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
 
@@ -1334,6 +1352,18 @@ destroy_Draw(void *drawptr)
     xfree(drawptr);
 }
 
+/**
+  * Get Draw object size.
+  *
+  * No Ruby usage (internal function)
+  *
+  * @param infoptr pointer to the Draw object
+  */
+static size_t
+Draw_memsize(const void *drawptr)
+{
+    return sizeof(Draw);
+}
 
 /**
  * Allocate & initialize a DrawOptions object.
@@ -1365,7 +1395,7 @@ DrawOptions_alloc(VALUE class)
 
     draw_options = ALLOC(Draw);
     memset(draw_options, 0, sizeof(Draw));
-    draw_options_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw_options);
+    draw_options_obj = TypedData_Wrap_Struct(class, &rm_draw_data_type, draw_options);
 
     RB_GC_GUARD(draw_options_obj);
 
@@ -1383,27 +1413,16 @@ DrawOptions_initialize(VALUE self)
 {
     Draw *draw_options;
 
-    Data_Get_Struct(self, Draw, draw_options);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw_options);
     draw_options->info = AcquireDrawInfo();
     if (!draw_options->info)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
     }
 
-    GetDrawInfo(NULL, draw_options->info);
-
     if (rb_block_given_p())
     {
-        if (rb_proc_arity(rb_block_proc()) == 0)
-        {
-            // Run the block in self's context
-            rb_warn("passing a block without an image argument is deprecated");
-            rb_obj_instance_eval(0, NULL, self);
-        }
-        else
-        {
-            rb_yield(self);
-        }
+        rb_yield(self);
     }
 
     return self;
@@ -1433,7 +1452,7 @@ PolaroidOptions_alloc(VALUE class)
     draw->info = CloneDrawInfo(image_info, (DrawInfo *) NULL);
     (void) DestroyImageInfo(image_info);
 
-    polaroid_obj = Data_Wrap_Struct(class, NULL, destroy_Draw, draw);
+    polaroid_obj = TypedData_Wrap_Struct(class, &rm_draw_data_type, draw);
 
     RB_GC_GUARD(polaroid_obj);
 
@@ -1455,7 +1474,7 @@ PolaroidOptions_initialize(VALUE self)
     ExceptionInfo *exception;
 
     // Default shadow color
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 
     exception = AcquireExceptionInfo();
     QueryColorCompliance("gray75", AllCompliance, &draw->shadow_color, exception);
@@ -1466,16 +1485,7 @@ PolaroidOptions_initialize(VALUE self)
 
     if (rb_block_given_p())
     {
-        if (rb_proc_arity(rb_block_proc()) == 0)
-        {
-            // Run the block in self's context
-            rb_warn("passing a block without an image argument is deprecated");
-            rb_obj_instance_eval(0, NULL, self);
-        }
-        else
-        {
-            rb_yield(self);
-        }
+        rb_yield(self);
     }
 
     return self;
@@ -1508,7 +1518,7 @@ PolaroidOptions_shadow_color_eq(VALUE self, VALUE shadow)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     Color_to_PixelColor(&draw->shadow_color, shadow);
     return shadow;
 }
@@ -1526,7 +1536,7 @@ PolaroidOptions_border_color_eq(VALUE self, VALUE border)
     Draw *draw;
 
     rb_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
     Color_to_PixelColor(&draw->info->border_color, border);
     return border;
 }
@@ -1575,6 +1585,9 @@ get_dummy_tm_img(VALUE klass)
 }
 
 
+// aliases for common use of structure types; GetMultilineTypeMetrics, GetTypeMetrics
+typedef GVL_STRUCT_TYPE(GetTypeMetrics) GVL_STRUCT_TYPE(get_type_metrics);
+
 /**
  * Call a get-type-metrics function.
  *
@@ -1592,11 +1605,7 @@ get_dummy_tm_img(VALUE klass)
  * @see Draw_get_multiline_type_metrics
  */
 static VALUE
-get_type_metrics(
-                int argc,
-                VALUE *argv,
-                VALUE self,
-                get_type_metrics_func_t getter)
+get_type_metrics(int argc, VALUE *argv, VALUE self, gvl_function_t fp)
 {
     Image *image;
     Draw *draw;
@@ -1604,7 +1613,7 @@ get_type_metrics(
     TypeMetric metrics;
     char *text = NULL;
     long text_l;
-    unsigned int okay;
+    MagickBooleanType okay;
 #if defined(IMAGEMAGICK_7)
     ExceptionInfo *exception;
 #endif
@@ -1613,7 +1622,7 @@ get_type_metrics(
     {
         case 1:                   // use default image
             text = rm_str2cstr(argv[0], &text_l);
-            Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
+            TypedData_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, &rm_image_data_type, image);
             break;
         case 2:
             t = rm_cur_image(argv[0]);
@@ -1630,7 +1639,7 @@ get_type_metrics(
         rb_raise(rb_eArgError, "no text to measure");
     }
 
-    Data_Get_Struct(self, Draw, draw);
+    TypedData_Get_Struct(self, Draw, &rm_draw_data_type, draw);
 #if defined(IMAGEMAGICK_7)
     exception = AcquireExceptionInfo();
     draw->info->text = InterpretImageProperties(NULL, image, text, exception);
@@ -1654,10 +1663,11 @@ get_type_metrics(
     }
 
 #if defined(IMAGEMAGICK_7)
-    okay = (*getter)(image, draw->info, &metrics, exception);
+    GVL_STRUCT_TYPE(get_type_metrics) args = { image, draw->info, &metrics, exception };
 #else
-    okay = (*getter)(image, draw->info, &metrics);
+    GVL_STRUCT_TYPE(get_type_metrics) args = { image, draw->info, &metrics };
 #endif
+    okay = (MagickBooleanType)CALL_FUNC_WITHOUT_GVL(fp, &args);
 
     magick_free(draw->info->text);
     draw->info->text = NULL;
