@@ -12,6 +12,11 @@
 
 #include "rmagick.h"
 
+static void GradientFill_free(void *fill);
+static size_t GradientFill_memsize(const void *ptr);
+static void TextureFill_free(void *fill_obj);
+static size_t TextureFill_memsize(const void *ptr);
+
 /** Data associated with a GradientFill */
 typedef struct
 {
@@ -29,18 +34,52 @@ typedef struct
     Image *texture; /**< the texture */
 } rm_TextureFill;
 
+const rb_data_type_t rm_gradient_fill_data_type = {
+    "Magick::GradientFill",
+    { NULL, GradientFill_free, GradientFill_memsize, },
+    0, 0,
+    RUBY_TYPED_FROZEN_SHAREABLE,
+};
+
+const rb_data_type_t rm_texture_fill_data_type = {
+    "Magick::TextureFill",
+    { NULL, TextureFill_free, TextureFill_memsize, },
+    0, 0,
+    RUBY_TYPED_FROZEN_SHAREABLE,
+};
+
+
+DEFINE_GVL_STUB2(SyncAuthenticPixels, Image *, ExceptionInfo *);
+
+
 /**
- * Free Fill or Fill subclass object (except for TextureFill).
+ * Free GradientFill or GradientFill subclass object (except for TextureFill).
  *
  * No Ruby usage (internal function)
  *
  * @param fill the fill
  */
-static void free_Fill(void *fill)
+static void
+GradientFill_free(void *fill)
 {
     xfree(fill);
 }
 
+
+/**
+  * Get GradientFill object size.
+  *
+  * No Ruby usage (internal function)
+  *
+  * @param ptr pointer to the GradientFill object
+  */
+static size_t
+GradientFill_memsize(const void *ptr)
+{
+    return sizeof(rm_GradientFill);
+}
+
+
 /**
  * Create new GradientFill object.
  *
@@ -51,7 +90,7 @@ GradientFill_alloc(VALUE class)
 {
     rm_GradientFill *fill;
 
-    return Data_Make_Struct(class, rm_GradientFill, NULL, free_Fill, fill);
+    return TypedData_Make_Struct(class, rm_GradientFill, &rm_gradient_fill_data_type, fill);
 }
 
 
@@ -78,7 +117,7 @@ GradientFill_initialize(
 {
     rm_GradientFill *fill;
 
-    Data_Get_Struct(self, rm_GradientFill, fill);
+    TypedData_Get_Struct(self, rm_GradientFill, &rm_gradient_fill_data_type, fill);
 
     fill->x1 = NUM2DBL(x1);
     fill->y1 = NUM2DBL(y1);
@@ -153,7 +192,8 @@ point_fill(
 #endif
         }
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         CHECK_EXCEPTION();
     }
 
@@ -222,7 +262,8 @@ vertical_fill(
             row_pixels += GetPixelChannels(image);
         }
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         CHECK_EXCEPTION();
     }
 
@@ -255,7 +296,8 @@ vertical_fill(
 
         memcpy(row_pixels, master, image->columns * sizeof(PixelPacket));
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         if (rm_should_raise_exception(exception, RetainExceptionRetention))
         {
             xfree((void *)master);
@@ -328,7 +370,8 @@ horizontal_fill(
             row_pixels += GetPixelChannels(image);
         }
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         CHECK_EXCEPTION();
     }
 
@@ -360,7 +403,8 @@ horizontal_fill(
 
         memcpy(col_pixels, master, image->rows * sizeof(PixelPacket));
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         if (rm_should_raise_exception(exception, RetainExceptionRetention))
         {
             xfree((void *)master);
@@ -471,7 +515,8 @@ v_diagonal_fill(
 #endif
         }
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         CHECK_EXCEPTION();
     }
 
@@ -577,7 +622,8 @@ h_diagonal_fill(
 #endif
         }
 
-        SyncAuthenticPixels(image, exception);
+        GVL_STRUCT_TYPE(SyncAuthenticPixels) args = { image, exception };
+        CALL_FUNC_WITHOUT_GVL(GVL_FUNC(SyncAuthenticPixels), &args);
         CHECK_EXCEPTION();
     }
 
@@ -599,7 +645,7 @@ GradientFill_fill(VALUE self, VALUE image_obj)
     PixelColor start_color, stop_color;
     double x1, y1, x2, y2;          // points on the line
 
-    Data_Get_Struct(self, rm_GradientFill, fill);
+    TypedData_Get_Struct(self, rm_GradientFill, &rm_gradient_fill_data_type, fill);
     image = rm_check_destroyed(image_obj);
 
     x1 = fill->x1;
@@ -662,7 +708,7 @@ GradientFill_fill(VALUE self, VALUE image_obj)
  * @param fill_obj the TextureFill
  */
 static void
-free_TextureFill(void *fill_obj)
+TextureFill_free(void *fill_obj)
 {
     rm_TextureFill *fill = (rm_TextureFill *)fill_obj;
 
@@ -674,6 +720,21 @@ free_TextureFill(void *fill_obj)
     xfree(fill);
 }
 
+
+/**
+  * Get TextureFill object size.
+  *
+  * No Ruby usage (internal function)
+  *
+  * @param ptr pointer to the TextureFill object
+  */
+static size_t
+TextureFill_memsize(const void *ptr)
+{
+    return sizeof(rm_TextureFill);
+}
+
+
 /**
  * Create new TextureFill object.
  *
@@ -683,7 +744,7 @@ VALUE
 TextureFill_alloc(VALUE class)
 {
     rm_TextureFill *fill;
-    return Data_Make_Struct(class, rm_TextureFill, NULL, free_TextureFill, fill);
+    return TypedData_Make_Struct(class, rm_TextureFill, &rm_texture_fill_data_type, fill);
 }
 
 /**
@@ -700,7 +761,7 @@ TextureFill_initialize(VALUE self, VALUE texture_arg)
     Image *texture;
     VALUE texture_image;
 
-    Data_Get_Struct(self, rm_TextureFill, fill);
+    TypedData_Get_Struct(self, rm_TextureFill, &rm_texture_fill_data_type, fill);
 
     texture_image = rm_cur_image(texture_arg);
 
@@ -732,7 +793,7 @@ TextureFill_fill(VALUE self, VALUE image_obj)
 #endif
 
     image = rm_check_destroyed(image_obj);
-    Data_Get_Struct(self, rm_TextureFill, fill);
+    TypedData_Get_Struct(self, rm_TextureFill, &rm_texture_fill_data_type, fill);
 
 #if defined(IMAGEMAGICK_7)
     exception = AcquireExceptionInfo();
