@@ -12,6 +12,16 @@
 
 #include "rmagick.h"
 
+static void Info_free(void *infoptr);
+static size_t Info_memsize(const void *infoptr);
+
+const rb_data_type_t rm_info_data_type = {
+    "Magick::Image::Info",
+    { NULL, Info_free, Info_memsize, },
+    0, 0,
+    RUBY_TYPED_FROZEN_SHAREABLE,
+};
+
 
 /**
  * Return the value of the specified option.
@@ -28,7 +38,7 @@ get_option(VALUE self, const char *key)
     Info *info;
     const char *value;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     value = GetImageOption(info, key);
     if (value)
@@ -54,7 +64,7 @@ set_option(VALUE self, const char *key, VALUE string)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(string))
     {
@@ -90,7 +100,7 @@ static VALUE set_color_option(VALUE self, const char *option, VALUE color)
     PixelColor pp;
     MagickBooleanType okay;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(color))
     {
@@ -136,7 +146,7 @@ static VALUE get_dbl_option(VALUE self, const char *option)
     double d;
     long n;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     value = GetImageOption(info, option);
     if (!value)
@@ -167,7 +177,7 @@ static VALUE set_dbl_option(VALUE self, const char *option, VALUE value)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(value))
     {
@@ -206,7 +216,7 @@ static VALUE set_dbl_option(VALUE self, const char *option, VALUE value)
 VALUE
 Info_antialias(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, antialias, boolean);
+    IMPLEMENT_TYPED_ATTR_READER(Info, antialias, boolean, &rm_info_data_type);
 }
 
 /**
@@ -218,7 +228,7 @@ Info_antialias(VALUE self)
 VALUE
 Info_antialias_eq(VALUE self, VALUE val)
 {
-    IMPLEMENT_ATTR_WRITER(Info, antialias, boolean);
+    IMPLEMENT_TYPED_ATTR_WRITER(Info, antialias, boolean, &rm_info_data_type);
 }
 
 /** Maximum length of a format (@see Info_aref) */
@@ -272,7 +282,7 @@ Info_aref(int argc, VALUE *argv, VALUE self)
 
     }
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     value = GetImageOption(info, fkey);
     if (!value)
     {
@@ -311,7 +321,7 @@ Info_aset(int argc, VALUE *argv, VALUE self)
     long format_l, key_l;
     char ckey[MaxTextExtent];
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     switch (argc)
     {
@@ -401,7 +411,7 @@ Info_authenticate(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 #if defined(IMAGEMAGICK_7)
     return C_str_to_R_str(GetImageOption(info, "authenticate"));
 #else
@@ -422,7 +432,7 @@ Info_authenticate_eq(VALUE self, VALUE passwd_arg)
     Info *info;
     char *passwd = NULL;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (!NIL_P(passwd_arg))
     {
@@ -465,7 +475,7 @@ Info_background_color(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return rm_pixelcolor_to_color_name_info(info, &info->background_color);
 }
 
@@ -481,7 +491,7 @@ Info_background_color_eq(VALUE self, VALUE bc_arg)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     Color_to_PixelColor(&info->background_color, bc_arg);
 
     return bc_arg;
@@ -498,7 +508,7 @@ Info_border_color(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return rm_pixelcolor_to_color_name_info(info, &info->border_color);
 }
 
@@ -513,7 +523,7 @@ Info_border_color_eq(VALUE self, VALUE bc_arg)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     Color_to_PixelColor(&info->border_color, bc_arg);
 
     return bc_arg;
@@ -572,7 +582,7 @@ Info_channel(int argc, VALUE *argv, VALUE self)
         raise_ChannelType_error(argv[argc-1]);
     }
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     info->channel = channels;
     return self;
@@ -589,7 +599,7 @@ Info_colorspace(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return ColorspaceType_find(info->colorspace);
 }
 
@@ -604,7 +614,7 @@ Info_colorspace_eq(VALUE self, VALUE colorspace)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     VALUE_TO_ENUM(colorspace, info->colorspace, ColorspaceType);
     return colorspace;
 }
@@ -640,7 +650,7 @@ Info_compression(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return CompressionType_find(info->compression);
 }
 
@@ -655,7 +665,7 @@ Info_compression_eq(VALUE self, VALUE type)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     VALUE_TO_ENUM(type, info->compression, CompressionType);
     return type;
 }
@@ -681,7 +691,7 @@ Info_define(int argc, VALUE *argv, VALUE self)
     unsigned int okay;
     VALUE fmt_arg;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     switch (argc)
     {
@@ -728,7 +738,7 @@ Info_delay(VALUE self)
     const char *delay;
     char *p;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     delay = GetImageOption(info, "delay");
     if (delay)
@@ -771,7 +781,7 @@ Info_delay_eq(VALUE self, VALUE string)
     Info *info;
     int not_num;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(string))
     {
@@ -803,7 +813,7 @@ Info_delay_eq(VALUE self, VALUE string)
 VALUE
 Info_density(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, density, str);
+    IMPLEMENT_TYPED_ATTR_READER(Info, density, str, &rm_info_data_type);
 }
 
 /**
@@ -820,7 +830,7 @@ Info_density_eq(VALUE self, VALUE density_arg)
     VALUE density;
     char *dens;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(density_arg))
     {
@@ -851,7 +861,7 @@ Info_density_eq(VALUE self, VALUE density_arg)
 VALUE
 Info_depth(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, depth, int);
+    IMPLEMENT_TYPED_ATTR_READER(Info, depth, int, &rm_info_data_type);
 }
 
 /**
@@ -866,7 +876,7 @@ Info_depth_eq(VALUE self, VALUE depth)
     Info *info;
     unsigned long d;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     d = NUM2ULONG(depth);
     switch (d)
     {
@@ -950,7 +960,7 @@ Info_dispose(VALUE self)
     ID dispose_id;
     const char *dispose;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     dispose_id = rb_intern("UndefinedDispose");
 
@@ -985,7 +995,7 @@ Info_dispose_eq(VALUE self, VALUE disp)
     const char *option;
     int x;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(disp))
     {
@@ -1017,7 +1027,7 @@ Info_dispose_eq(VALUE self, VALUE disp)
 VALUE
 Info_dither(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, dither, boolean);
+    IMPLEMENT_TYPED_ATTR_READER(Info, dither, boolean, &rm_info_data_type);
 }
 
 /**
@@ -1029,7 +1039,7 @@ Info_dither(VALUE self)
 VALUE
 Info_dither_eq(VALUE self, VALUE val)
 {
-    IMPLEMENT_ATTR_WRITER(Info, dither, boolean);
+    IMPLEMENT_TYPED_ATTR_WRITER(Info, dither, boolean, &rm_info_data_type);
 }
 
 
@@ -1043,7 +1053,7 @@ Info_endian(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return EndianType_find(info->endian);
 }
 
@@ -1065,7 +1075,7 @@ Info_endian_eq(VALUE self, VALUE endian)
         VALUE_TO_ENUM(endian, type, EndianType);
     }
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     info->endian = type;
     return endian;
 }
@@ -1080,7 +1090,7 @@ Info_endian_eq(VALUE self, VALUE endian)
 VALUE
 Info_extract(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, extract, str);
+    IMPLEMENT_TYPED_ATTR_READER(Info, extract, str, &rm_info_data_type);
 }
 
 /**
@@ -1097,7 +1107,7 @@ Info_extract_eq(VALUE self, VALUE extract_arg)
     char *extr;
     VALUE extract;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(extract_arg))
     {
@@ -1133,7 +1143,7 @@ Info_filename(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return rb_str_new2(info->filename);
 }
 
@@ -1150,7 +1160,7 @@ Info_filename_eq(VALUE self, VALUE filename)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     // Allow "nil" - remove current filename
     if (NIL_P(filename) || StringValueCStr(filename) == NULL)
@@ -1201,7 +1211,7 @@ Info_fill_eq(VALUE self, VALUE color)
 VALUE
 Info_font(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, font, str);
+    IMPLEMENT_TYPED_ATTR_READER(Info, font, str, &rm_info_data_type);
 }
 
 /**
@@ -1215,7 +1225,7 @@ Info_font_eq(VALUE self, VALUE font_arg)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     if (NIL_P(font_arg) || StringValueCStr(font_arg) == NULL)
     {
         magick_free(info->font);
@@ -1240,7 +1250,7 @@ VALUE Info_format(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     if (*info->magick)
     {
         const MagickInfo *magick_info;
@@ -1270,7 +1280,7 @@ Info_format_eq(VALUE self, VALUE magick)
     char *mgk;
     ExceptionInfo *exception;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     mgk = StringValueCStr(magick);
 
@@ -1297,7 +1307,7 @@ Info_format_eq(VALUE self, VALUE magick)
 VALUE
 Info_fuzz(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, fuzz, dbl);
+    IMPLEMENT_TYPED_ATTR_READER(Info, fuzz, dbl, &rm_info_data_type);
 }
 
 /**
@@ -1313,7 +1323,7 @@ Info_fuzz_eq(VALUE self, VALUE fuzz)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     info->fuzz = rm_fuzz_to_dbl(fuzz);
     return fuzz;
 }
@@ -1380,7 +1390,7 @@ VALUE Info_gravity(VALUE self)
     const char *gravity;
     ID gravity_id;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     gravity_id = rb_intern("UndefinedGravity");
 
@@ -1416,7 +1426,7 @@ Info_gravity_eq(VALUE self, VALUE grav)
     const char *option;
     int x;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(grav))
     {
@@ -1451,7 +1461,7 @@ Info_image_type(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return ImageType_find(info->type);
 }
 
@@ -1466,7 +1476,7 @@ Info_image_type_eq(VALUE self, VALUE type)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     VALUE_TO_ENUM(type, info->type, ImageType);
     return type;
 }
@@ -1481,7 +1491,7 @@ Info_interlace(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return InterlaceType_find(info->interlace);
 }
 
@@ -1496,7 +1506,7 @@ Info_interlace_eq(VALUE self, VALUE inter)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     VALUE_TO_ENUM(inter, info->interlace, InterlaceType);
     return inter;
 }
@@ -1533,7 +1543,7 @@ Info_matte_color(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return rm_pixelcolor_to_color_name_info(info, &info->matte_color);
 }
 
@@ -1548,38 +1558,12 @@ Info_matte_color_eq(VALUE self, VALUE matte_arg)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     Color_to_PixelColor(&info->matte_color, matte_arg);
 
     return matte_arg;
 }
 
-/**
- * Establish a progress monitor.
- *
- * @param monitor [Proc] the monitor
- * @return [Proc] monitor
- * @see Image#monitor=
- */
-VALUE
-Info_monitor_eq(VALUE self, VALUE monitor)
-{
-    Info *info;
-
-    Data_Get_Struct(self, Info, info);
-
-    if (NIL_P(monitor))
-    {
-        info->progress_monitor = NULL;
-    }
-    else
-    {
-        SetImageInfoProgressMonitor(info, rm_progress_monitor, (void *)monitor);
-    }
-
-    return monitor;
-}
-
 /**
  * Get the monochrome value.
  *
@@ -1588,7 +1572,7 @@ Info_monitor_eq(VALUE self, VALUE monitor)
 VALUE
 Info_monochrome(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, monochrome, boolean);
+    IMPLEMENT_TYPED_ATTR_READER(Info, monochrome, boolean, &rm_info_data_type);
 }
 
 /**
@@ -1600,7 +1584,7 @@ Info_monochrome(VALUE self)
 VALUE
 Info_monochrome_eq(VALUE self, VALUE val)
 {
-    IMPLEMENT_ATTR_WRITER(Info, monochrome, boolean);
+    IMPLEMENT_TYPED_ATTR_WRITER(Info, monochrome, boolean, &rm_info_data_type);
 }
 
 /**
@@ -1611,7 +1595,7 @@ Info_monochrome_eq(VALUE self, VALUE val)
 VALUE
 Info_number_scenes(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, number_scenes, ulong);
+    IMPLEMENT_TYPED_ATTR_READER(Info, number_scenes, ulong, &rm_info_data_type);
 }
 
 /**
@@ -1623,7 +1607,7 @@ Info_number_scenes(VALUE self)
 VALUE
 Info_number_scenes_eq(VALUE self, VALUE val)
 {
-    IMPLEMENT_ATTR_WRITER(Info, number_scenes, ulong);
+    IMPLEMENT_TYPED_ATTR_WRITER(Info, number_scenes, ulong, &rm_info_data_type);
 }
 
 /**
@@ -1636,7 +1620,7 @@ Info_orientation(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return OrientationType_find(info->orientation);
 }
 
@@ -1652,7 +1636,7 @@ Info_orientation_eq(VALUE self, VALUE inter)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     VALUE_TO_ENUM(inter, info->orientation, OrientationType);
     return inter;
 }
@@ -1670,7 +1654,7 @@ Info_origin(VALUE self)
     Info *info;
     const char *origin;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     origin = GetImageOption(info, "origin");
     return origin ? rb_str_new2(origin) : Qnil;
@@ -1695,7 +1679,7 @@ Info_origin_eq(VALUE self, VALUE origin_arg)
     VALUE origin_str;
     char *origin;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(origin_arg))
     {
@@ -1731,7 +1715,7 @@ Info_page(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return info->page ? rb_str_new2(info->page) : Qnil;
 
 }
@@ -1751,7 +1735,7 @@ Info_page_eq(VALUE self, VALUE page_arg)
     VALUE geom_str;
     char *geometry;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     if (NIL_P(page_arg))
     {
         magick_free(info->page);
@@ -1781,7 +1765,7 @@ Info_page_eq(VALUE self, VALUE page_arg)
 VALUE
 Info_pointsize(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, pointsize, dbl);
+    IMPLEMENT_TYPED_ATTR_READER(Info, pointsize, dbl, &rm_info_data_type);
 }
 
 /**
@@ -1793,7 +1777,7 @@ Info_pointsize(VALUE self)
 VALUE
 Info_pointsize_eq(VALUE self, VALUE val)
 {
-    IMPLEMENT_ATTR_WRITER(Info, pointsize, dbl);
+    IMPLEMENT_TYPED_ATTR_WRITER(Info, pointsize, dbl, &rm_info_data_type);
 }
 
 /**
@@ -1804,7 +1788,7 @@ Info_pointsize_eq(VALUE self, VALUE val)
 VALUE
 Info_quality(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, quality, ulong);
+    IMPLEMENT_TYPED_ATTR_READER(Info, quality, ulong, &rm_info_data_type);
 }
 
 /**
@@ -1816,7 +1800,7 @@ Info_quality(VALUE self)
 VALUE
 Info_quality_eq(VALUE self, VALUE val)
 {
-    IMPLEMENT_ATTR_WRITER(Info, quality, ulong);
+    IMPLEMENT_TYPED_ATTR_WRITER(Info, quality, ulong, &rm_info_data_type);
 }
 
 /**
@@ -1829,7 +1813,7 @@ Info_sampling_factor(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     if (info->sampling_factor)
     {
         return rb_str_new2(info->sampling_factor);
@@ -1853,7 +1837,7 @@ Info_sampling_factor_eq(VALUE self, VALUE sampling_factor)
     char *sampling_factor_p = NULL;
     long sampling_factor_len = 0;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (!NIL_P(sampling_factor))
     {
@@ -1884,7 +1868,7 @@ Info_scene(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return  ULONG2NUM(info->scene);
 }
 
@@ -1901,7 +1885,7 @@ Info_scene_eq(VALUE self, VALUE scene)
     Info *info;
     char buf[25];
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     info->scene = NUM2ULONG(scene);
 
     snprintf(buf, sizeof(buf), "%"RMIuSIZE"", info->scene);
@@ -1919,7 +1903,7 @@ Info_scene_eq(VALUE self, VALUE scene)
 VALUE
 Info_server_name(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, server_name, str);
+    IMPLEMENT_TYPED_ATTR_READER(Info, server_name, str, &rm_info_data_type);
 }
 
 
@@ -1934,7 +1918,7 @@ Info_server_name_eq(VALUE self, VALUE server_arg)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     if (NIL_P(server_arg) || StringValueCStr(server_arg) == NULL)
     {
         magick_free(info->server_name);
@@ -1959,7 +1943,7 @@ Info_server_name_eq(VALUE self, VALUE server_arg)
 VALUE
 Info_size(VALUE self)
 {
-    IMPLEMENT_ATTR_READER(Info, size, str);
+    IMPLEMENT_TYPED_ATTR_READER(Info, size, str, &rm_info_data_type);
 }
 
 /**
@@ -1976,7 +1960,7 @@ Info_size_eq(VALUE self, VALUE size_arg)
     VALUE size;
     char *sz;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (NIL_P(size_arg))
     {
@@ -2062,7 +2046,7 @@ Info_texture_eq(VALUE self, VALUE texture)
     Image *image;
     char name[MaxTextExtent];
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     // Delete any existing texture file
     if (info->texture)
@@ -2100,7 +2084,7 @@ Info_tile_offset(VALUE self)
     Info *info;
     const char *tile_offset;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     tile_offset = GetImageOption(info, "tile-offset");
 
@@ -2134,7 +2118,7 @@ Info_tile_offset_eq(VALUE self, VALUE offset)
         rb_raise(rb_eArgError, "invalid tile offset geometry: %s", tile_offset);
     }
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     DeleteImageOption(info, "tile-offset");
     SetImageOption(info, "tile-offset", tile_offset);
@@ -2156,7 +2140,7 @@ Info_transparent_color(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return rm_pixelcolor_to_color_name_info(info, &info->transparent_color);
 }
 
@@ -2172,7 +2156,7 @@ Info_transparent_color_eq(VALUE self, VALUE tc_arg)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     Color_to_PixelColor(&info->transparent_color, tc_arg);
 
     return tc_arg;
@@ -2204,7 +2188,7 @@ Info_undefine(VALUE self, VALUE format, VALUE key)
 
     snprintf(fkey, sizeof(fkey), "%.60s:%.*s", format_p, (int)(MaxTextExtent-61), key_p);
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     DeleteImageOption(info, fkey);
 
     return self;
@@ -2244,7 +2228,7 @@ Info_units(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     return ResolutionType_find(info->units);
 }
 
@@ -2259,7 +2243,7 @@ Info_units_eq(VALUE self, VALUE units)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
     VALUE_TO_ENUM(units, info->units, ResolutionType);
     return units;
 }
@@ -2274,7 +2258,7 @@ Info_view(VALUE self)
 {
     Info *info;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 #if defined(IMAGEMAGICK_7)
     return C_str_to_R_str(GetImageOption(info, "fpx:view"));
 #else
@@ -2294,7 +2278,7 @@ Info_view_eq(VALUE self, VALUE view_arg)
     Info *info;
     char *view = NULL;
 
-    Data_Get_Struct(self, Info, info);
+    TypedData_Get_Struct(self, Info, &rm_info_data_type, info);
 
     if (!NIL_P(view_arg))
     {
@@ -2334,7 +2318,7 @@ Info_view_eq(VALUE self, VALUE view_arg)
  * @param infoptr pointer to the Info object
  */
 static void
-destroy_Info(void *infoptr)
+Info_free(void *infoptr)
 {
     Info *info = (Info *)infoptr;
 
@@ -2348,6 +2332,18 @@ destroy_Info(void *infoptr)
     DestroyImageInfo(info);
 }
 
+/**
+  * Get Info object size.
+  *
+  * No Ruby usage (internal function)
+  *
+  * @param infoptr pointer to the Info object
+  */
+static size_t
+Info_memsize(const void *infoptr)
+{
+    return sizeof(Info);
+}
 
 /**
  * Create an Image::Info object.
@@ -2368,7 +2364,7 @@ Info_alloc(VALUE class)
     {
         rb_raise(rb_eNoMemError, "not enough memory to initialize Info object");
     }
-    info_obj = Data_Wrap_Struct(class, NULL, destroy_Info, info);
+    info_obj = TypedData_Wrap_Struct(class, &rm_info_data_type, info);
 
     RB_GC_GUARD(info_obj);
 
@@ -2414,16 +2410,7 @@ Info_initialize(VALUE self)
 {
     if (rb_block_given_p())
     {
-        if (rb_proc_arity(rb_block_proc()) == 0)
-        {
-            // Run the block in self's context
-            rb_warn("passing a block without an image argument is deprecated");
-            rb_obj_instance_eval(0, NULL, self);
-        }
-        else
-        {
-            rb_yield(self);
-        }
+        rb_yield(self);
     }
     return self;
 }
