@@ -234,6 +234,10 @@ static void set_managed_memory(void)
 void
 Init_RMagick2(void)
 {
+#ifdef HAVE_RB_EXT_RACTOR_SAFE
+    rb_ext_ractor_safe(true);
+#endif
+
     Module_Magick = rb_define_module("Magick");
 
     set_managed_memory();
@@ -242,13 +246,10 @@ Init_RMagick2(void)
 
     test_Magick_version();
 
-    rm_main_thread_id = rm_current_thread_id();
-
     /*-----------------------------------------------------------------------*/
     /* Create IDs for frequently used methods, etc.                          */
     /*-----------------------------------------------------------------------*/
 
-    rm_ID_trace_proc       = rb_intern("@trace_proc");
     rm_ID_call             = rb_intern("call");
     rm_ID_changed          = rb_intern("changed");
     rm_ID_cur_image        = rb_intern("cur_image");
@@ -358,7 +359,6 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "matte_color=", Image_matte_color_eq, 1);
     rb_define_method(Class_Image, "mean_error_per_pixel", Image_mean_error_per_pixel, 0);
     rb_define_method(Class_Image, "mime_type", Image_mime_type, 0);
-    rb_define_method(Class_Image, "monitor=", Image_monitor_eq, 1);
     rb_define_method(Class_Image, "montage", Image_montage, 0);
     rb_define_method(Class_Image, "normalized_mean_error", Image_normalized_mean_error, 0);
     rb_define_method(Class_Image, "normalized_maximum_error", Image_normalized_maximum_error, 0);
@@ -620,7 +620,7 @@ Init_RMagick2(void)
     rb_define_method(Class_Image, "write", Image_write, 1);
 
     /*-----------------------------------------------------------------------*/
-    /* Class Magick::ImageList methods (see also RMagick.rb)                 */
+    /* Class Magick::ImageList methods                                       */
     /*-----------------------------------------------------------------------*/
 
     Class_ImageList = rb_define_class_under(Module_Magick, "ImageList", rb_cObject);
@@ -735,8 +735,7 @@ Init_RMagick2(void)
     // include Comparable
     rb_include_module(Class_Pixel, rb_mComparable);
 
-    // Magick::Pixel has 4 constructors: "new" "from_color", "from_hsla",
-    // and the deprecated "from_HSL".
+    // Magick::Pixel has 3 constructors: "new" "from_color", "from_hsla"
     rb_define_alloc_func(Class_Pixel, Pixel_alloc);
     rb_define_singleton_method(Class_Pixel, "from_color", Pixel_from_color, 1);
     rb_define_singleton_method(Class_Pixel, "from_hsla", Pixel_from_hsla, -1);
@@ -878,7 +877,6 @@ Init_RMagick2(void)
     rb_define_method(Class_Info, "label=", Info_label_eq, 1);
     rb_define_method(Class_Info, "matte_color", Info_matte_color, 0);
     rb_define_method(Class_Info, "matte_color=", Info_matte_color_eq, 1);
-    rb_define_method(Class_Info, "monitor=", Info_monitor_eq, 1);
     rb_define_method(Class_Info, "monochrome", Info_monochrome, 0);
     rb_define_method(Class_Info, "monochrome=", Info_monochrome_eq, 1);
     rb_define_method(Class_Info, "number_scenes", Info_number_scenes, 0);
@@ -1058,6 +1056,13 @@ Init_RMagick2(void)
 #if defined(IMAGEMAGICK_GREATER_THAN_EQUAL_6_8_9)
         ENUMERATOR(AssociateAlphaChannel)
         ENUMERATOR(DisassociateAlphaChannel)
+#endif
+#if defined(IMAGEMAGICK_7)
+        ENUMERATOR(OnAlphaChannel)
+        ENUMERATOR(OffAlphaChannel)
+#else
+        ENUMERATORV(OnAlphaChannel, ActivateAlphaChannel)
+        ENUMERATORV(OffAlphaChannel, DeactivateAlphaChannel)
 #endif
     END_ENUM
 
@@ -1852,7 +1857,7 @@ Init_RMagick2(void)
  *
  * Notes:
  *   - Bypass the test by defining the constant RMAGICK_BYPASS_VERSION_TEST to
- *     'true' at the top level, before requiring 'RMagick'
+ *     'true' at the top level, before requiring 'rmagick'
  */
 static void
 test_Magick_version(void)
@@ -1919,8 +1924,7 @@ version_constants(void)
             "This is %s ($Date: 2009/12/20 02:33:33 $) Copyright (C) 2009 by Timothy P. Hunter\n"
             "Built with %s\n"
             "Built for %s\n"
-            "Web page: http://rmagick.rubyforge.org\n"
-            "Email: rmagick@rubyforge.org\n",
+            "Web page: https://rmagick.github.io/\n",
             Q(RMAGICK_VERSION_STRING), mgk_version, Q(RUBY_VERSION_STRING));
 
     str = rb_str_new2(long_version);
