@@ -22,6 +22,9 @@ static VALUE rescue_not_str(VALUE, VALUE ATTRIBUTE_UNUSED) ATTRIBUTE_NORETURN;
 static void handle_exception(ExceptionInfo *, Image *, ErrorRetention);
 
 
+DEFINE_GVL_STUB5(CloneImage, const Image *, const size_t, const size_t, const MagickBooleanType, ExceptionInfo *);
+
+
 /**
  * ImageMagick safe version of malloc.
  *
@@ -269,7 +272,7 @@ rm_check_destroyed(VALUE obj)
 {
     Image *image;
 
-    Data_Get_Struct(obj, Image, image);
+    TypedData_Get_Struct(obj, Image, &rm_image_data_type, image);
     if (!image)
     {
         rb_raise(Class_DestroyedImageError, "destroyed image");
@@ -780,7 +783,7 @@ rm_write_temp_image(Image *image, char *temp_name, size_t temp_name_l)
     snprintf(temp_name, temp_name_l, "mpri:%d", id);
 
     // Omit "mpri:" from filename to form the key
-    okay = SetImageRegistry(ImageRegistryType, temp_name+5, image, exception);
+    okay = (MagickBooleanType)SetImageRegistry(ImageRegistryType, temp_name+5, image, exception);
     CHECK_EXCEPTION();
     DestroyExceptionInfo(exception);
     if (!okay)
@@ -1025,15 +1028,7 @@ rm_get_optional_arguments(VALUE img)
         argv[0] = img;
         opt_args = rb_class_new_instance(1, argv, optional_method_arguments);
 
-        if (rb_proc_arity(rb_block_proc()) == 0)
-        {
-            rb_warn("passing a block without an image argument is deprecated");
-            rb_obj_instance_eval(0, NULL, opt_args);
-        }
-        else
-        {
-            rb_yield(opt_args);
-        }
+        rb_yield(opt_args);
     }
 
     RB_GC_GUARD(optional_method_arguments);
@@ -1273,7 +1268,7 @@ void rm_sync_image_options(Image *image, Info *info)
 /**
  * Replicate old (ImageMagick < 6.3.2) EXIF:* functionality using
  * GetImageProperty by returning the exif entries as a single string, separated
- * by \n's.  Do this so that RMagick.rb works no matter which version of
+ * by \n's.  Do this so that RMagick works no matter which version of
  * ImageMagick is in use.
  *
  * No Ruby usage (internal function)
@@ -1392,7 +1387,7 @@ rm_exif_by_entry(Image *image)
 /**
  * Replicate old (ImageMagick < 6.3.2) EXIF:! functionality using
  * GetImageProperty by returning the exif entries as a single string, separated
- * by \n's. Do this so that RMagick.rb works no matter which version of
+ * by \n's. Do this so that RMagick works no matter which version of
  * ImageMagick is in use.
  *
  * No Ruby usage (internal function)
@@ -1526,7 +1521,8 @@ rm_clone_image(Image *image)
     ExceptionInfo *exception;
 
     exception = AcquireExceptionInfo();
-    clone = CloneImage(image, 0, 0, MagickTrue, exception);
+    GVL_STRUCT_TYPE(CloneImage) args = { image, 0, 0, MagickTrue, exception };
+    clone = (Image *)CALL_FUNC_WITHOUT_GVL(GVL_FUNC(CloneImage), &args);
     if (!clone)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
@@ -1538,64 +1534,6 @@ rm_clone_image(Image *image)
 }
 
 
-/**
- * SetImage(Info)ProgressMonitor exit.
- *
- * No Ruby usage (internal function)
- *
- * Notes:
- *   - ImageMagick's "tag" argument is unused. We pass along the method name
- *     instead.
- *
- * @param tag ImageMagick argument (unused)
- * @param of the offset type
- * @param sp the size type
- * @param client_data pointer to the progress method to call
- * @return true if calling client_data returns a non-nil value, otherwise false
- */
-MagickBooleanType
-rm_progress_monitor(
-    const char *tag ATTRIBUTE_UNUSED,
-    const MagickOffsetType of,
-    const MagickSizeType sp,
-    void *client_data)
-{
-    VALUE rval;
-    VALUE method, offset, span;
-
-    // Check running thread.
-    if (rm_current_thread_id() != rm_main_thread_id)
-    {
-        // ImageMagick might call back in a different thread than Ruby is running in.
-        // If it is a different thread, it would not have a Ruby GVL and
-        // it could not retrieve properly Ruby stack.
-
-        // Unfortunately, there is no API available to check if the current thread has a GVL,
-        // so the thread id was checked in here.
-        return MagickTrue;
-    }
-
-#if defined(HAVE_LONG_LONG)     // defined in Ruby's defines.h
-    offset = rb_ll2inum(of);
-    span = rb_ull2inum(sp);
-#else
-    offset = rb_int2big((long)of);
-    span = rb_uint2big((unsigned long)sp);
-#endif
-
-    method = rb_id2str(rb_frame_this_func());
-
-    rval = rb_funcall((VALUE)client_data, rm_ID_call, 3, method, offset, span);
-
-    RB_GC_GUARD(rval);
-    RB_GC_GUARD(method);
-    RB_GC_GUARD(offset);
-    RB_GC_GUARD(span);
-
-    return RTEST(rval) ? MagickTrue : MagickFalse;
-}
-
-
 /**
  * Remove the ImageMagick links between images in an scene sequence.
  *
@@ -1901,20 +1839,3 @@ rm_raise_exception(ExceptionInfo *exception)
 
     rm_magick_error(msg);
 }
-
-/**
- * Get current thread id.
- *
- * No Ruby usage (internal function)
- *
- * @return thread id
- */
-unsigned long long
-rm_current_thread_id()
-{
-#if defined(_WIN32)
-    return (unsigned long long)GetCurrentThreadId();
-#else
-    return (unsigned long long)pthread_self();
-#endif
-}
