@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 # $Id: RMagick.rb,v 1.84 2009/09/15 22:08:41 rmagick Exp $
 #==============================================================================
 #                  Copyright (C) 2009 by Timothy P. Hunter
@@ -12,7 +14,7 @@ if RUBY_PLATFORM =~ /mingw/i
   begin
     require 'ruby_installer'
     ENV['PATH'].split(File::PATH_SEPARATOR).grep(/ImageMagick/i).each do |path|
-      RubyInstaller::Runtime.add_dll_directory(path)
+      RubyInstaller::Runtime.add_dll_directory(path) if File.exist?(File.join(path, 'CORE_RL_magick_.dll')) || File.exist?(File.join(path, 'CORE_RL_MagickCore_.dll'))
     end
   rescue LoadError
   end
@@ -23,9 +25,6 @@ require 'observer'
 require 'RMagick2.so'
 
 module Magick
-  @formats = nil
-  @trace_proc = nil
-  @exit_block_set_up = nil
   IMAGEMAGICK_VERSION = Magick::Magick_version.split[1].split('-').first
 
   class << self
@@ -53,45 +52,13 @@ module Magick
     #   => {"3FR"=>" r-+", "3G2"=>" r-+", "3GP"=>" r-+", "A"=>"*rw+",
     #   ...
     def formats
-      @formats ||= init_formats
+      formats = init_formats
 
       if block_given?
-        @formats.each { |k, v| yield k, v }
+        formats.each { |k, v| yield k, v }
         self
       else
-        @formats
-      end
-    end
-
-    # If the Magick module attribute +trace_proc+ is set to a Proc object,
-    # RMagick calls the proc whenever an image is created or destroyed.
-    #
-    # You can use this proc to keep track of which images your program has created
-    # and which have been destroyed.
-    #
-    # @param p [Proc] The proc object.
-    #   The following value will be passed into the proc object.
-    #   - +which+ - A symbol that indicates which operation the proc is being called for.
-    #     If the proc is called for an image creation, the value is +:c+.
-    #     If called for an image destruction, the value is +:d+.
-    #   - +description+ - A string describing the image. This is the same string that
-    #     would be returned by calling the image's inspect method.
-    #   - +id+ - A unique identifier for the image. This identifier is not the same as the object's +object_id+.
-    #   - +method+ - The name of the method responsible for creating or destroying the image.
-    #
-    # @example
-    #   Magick.trace_proc = proc do |which, description, id, method|
-    #     ...
-    #   end
-    def trace_proc=(p)
-      m = Mutex.new
-      m.synchronize do
-        if @trace_proc.nil? && !p.nil? && !@exit_block_set_up
-          at_exit { @trace_proc = nil }
-          @exit_block_set_up = true
-        end
-
-        @trace_proc = p
+        formats
       end
     end
   end
@@ -109,7 +76,7 @@ module Magick
   MinimumGeometry  = GeometryValue.new(:MinimumGeometry, 6).freeze
 
   class Geometry
-    FLAGS = ['', '%', '!', '<', '>', '@', '^']
+    FLAGS = ['', '%', '!', '<', '>', '@', '^'].freeze
     RFLAGS = {
       '%' => PercentGeometry,
       '!' => AspectGeometry,
@@ -117,7 +84,7 @@ module Magick
       '>' => GreaterGeometry,
       '@' => AreaGeometry,
       '^' => MinimumGeometry
-    }
+    }.freeze
 
     attr_accessor :width, :height, :x, :y, :flag
 
@@ -173,7 +140,7 @@ module Magick
 
     # Convert object to a geometry string
     def to_s
-      str = ''
+      str = String.new
       if @width > 0
         fmt = @width.truncate == @width ? '%d' : '%.2f'
         str << sprintf(fmt, @width)
@@ -804,12 +771,6 @@ module Magick
     module Post_ObjectData_Descriptor
       Confirmed_ObjectData_Size              = '9:10'
     end
-
-    # Make all constants above immutable
-    constants.each do |record|
-      rec = const_get(record)
-      rec.constants.each { |ds| rec.const_get(ds).freeze }
-    end
   end # module Magick::IPTC
 
   # Ruby-level Magick::Image methods
